#!/bin/bash

if [[ $EUID -ne 0 ]]; then
   echo "This script must be run as root"
   exit 1
fi

PARAMS=( "$@" )
UPDATE_SERVER="update.2x.mikbill.pro"
TESTING=0
FORCE=0

while [[ $# -gt 0 ]]; do
  key="$1"

  case $key in
    -t|--testing)
      # shellcheck disable=SC2034
      TESTRING=1
      UPDATE_SERVER="current.2x.mikbill.pro"
	  shift
      ;;
    -f|--force)
      FORCE=1
	  shift
      ;;
    *)
      # unknown option
	  shift
      ;;
  esac
done

LICENSE_DIR="/var/mikbill/license"
if [ ! -d "${LICENSE_DIR}" ]; then
  mkdir -p "${LICENSE_DIR}"
fi

URL="http://free.${UPDATE_SERVER}"
if [ "$(ls -A "${LICENSE_DIR}")" ]; then
  URL="http://pay.${UPDATE_SERVER}"
fi

FILE_NAME="stat"
MODULE="stat"
BILL_HOME="/var/mikbill"
WORK_DIR="/var/www/mikbill"
UPD_DIR="${BILL_HOME}/updates/${MODULE}"
BKP_DIR="${BILL_HOME}/www_backups"
TMP_DIR="/tmp/mikbill"

UPDATER_REVISION=2
UPDATER_NAME="stat_update.sh"

DATENAME=$(date +%F_%H-%M)
INFO="[${DATENAME} ${VERSION}]"
LOG_DIR="/var/log/mikbill/${MODULE}"
FILE_LOG="${LOG_DIR}/update.log"

function upd_log() {
  echo "${INFO} $1" >> "${FILE_LOG}"
}

if [ ! -d "${TMP_DIR}" ]; then
  upd_log "create tmp ${TMP_DIR}"
  mkdir -p "${TMP_DIR}"
fi

if [ ! -d "${LOG_DIR}" ]; then
  upd_log "create log dir ${LOG_DIR}"
  mkdir -p "${LOG_DIR}"
fi

if [ ! -d "${BKP_DIR}" ]; then
  upd_log "create backup dir ${BKP_DIR}"
  mkdir -p "${BKP_DIR}"
fi

if [ ! -d "${UPD_DIR}" ]; then
  upd_log "create update dir ${UPD_DIR}"
  mkdir -p "${UPD_DIR}"
fi

if [ ! -d "${WORK_DIR}" ]; then
  upd_log "${WORK_DIR} dir not found"
  exit 0
fi

if [ $TESTING -eq 1 ]; then
    upd_log "use testing update server"
fi

cd "${UPD_DIR}" || exit
upd_log "check version UPDATER SOFT"
SITE_REVISION=$(wget -qO- "${URL}/${UPDATER_NAME}.revision")
if [ -n "${SITE_REVISION}" ]; then
  if [ "${SITE_REVISION}" != "${UPDATER_REVISION}" ]; then
    upd_log "found new version for UPDATER SOFT"
    wget -q -O "${UPD_DIR}/${UPDATER_NAME}.new" "${URL}/${UPDATER_NAME}"
    if [ $? -ne 0 ]; then
      upd_log "download FAILED! please check DNS and connection to ${URL}/${UPDATER_NAME}"
      if [ -f "${UPD_DIR}/${UPDATER_NAME}.new" ]; then
        rm -f "${UPD_DIR}/${UPDATER_NAME}.new"
      fi
      exit 0
    fi

    upd_log "run NEW VERSION UPDATER SOFT"
    if [ -f "${UPD_DIR}/${UPDATER_NAME}" ]; then
      rm -f "${UPD_DIR}/${UPDATER_NAME}"
    fi

    mv "${UPD_DIR}/${UPDATER_NAME}.new" "${UPD_DIR}/${UPDATER_NAME}"
    chmod +x "${UPD_DIR}/${UPDATER_NAME}"
    bash "${UPD_DIR}/${UPDATER_NAME}" "${PARAMS[@]}"
    exit 0
  else
    upd_log "installed latest version UPDATER SOFT"
  fi
fi

wget -O "${FILE_NAME}.downloaded.checksum" -q "${URL}/${FILE_NAME}.checksum"
if [ $? -ne 0 ]; then
  upd_log "download FAILED! please check DNS and connection to ${URL}/${FILE_NAME}.checksum"
  if [ -f "${FILE_NAME}.downloaded.checksum" ]; then
    rm -f "${FILE_NAME}.downloaded.checksum"
  fi
  exit 0
fi

if [ ! -f "${FILE_NAME}.current.checksum" ]; then
  echo 0 >"${FILE_NAME}.current.checksum"
fi

if [ "$FORCE" -eq 1 ]; then
	upd_log "force mode"
fi

MD5_CURRENT=$(cat "${FILE_NAME}.current.checksum")
MD5_DOWNLOADED=$(cat "${FILE_NAME}.downloaded.checksum")
if [ "${MD5_CURRENT}" != "${MD5_DOWNLOADED}" ] || [ "$FORCE" -eq 1 ]; then
  upd_log "download ${FILE_NAME} update..."
  cd "${TMP_DIR}" || exit

  wget -O "${FILE_NAME}.tar.gz" -q "${URL}/${FILE_NAME}.tar.gz"
  if [ $? -ne 0 ]; then
    upd_log "download FAILED! please check DNS and connection to ${URL}/${FILE_NAME}"
    if [ -f "${FILE_NAME}.tar.gz" ]; then
      rm -f "${FILE_NAME}.tar.gz"
    fi
    exit 0
  fi

  if [ $TESTING -eq 0 ]; then
    echo "${MD5_DOWNLOADED}" > "${UPD_DIR}/${FILE_NAME}.current.checksum"
  fi

  mv "${FILE_NAME}.tar.gz" "${WORK_DIR}/"
  cd "${WORK_DIR}" || exit

  if [ -d "${MODULE}" ]; then
    upd_log "backup current version to ${BKP_DIR}/${FILE_NAME}_${VERSION}.${DATENAME}.tar.gz"
    tar -zcf "${BKP_DIR}/${FILE_NAME}_${VERSION}.${DATENAME}.tar.gz" "${MODULE}/"
  fi

  if [ -f "${FILE_NAME}.tar.gz" ]; then
    upd_log "install new ${FILE_NAME} update"
    tar -zxf "${FILE_NAME}.tar.gz"
    rm -f "${FILE_NAME}.tar.gz"
  fi
else
  upd_log "installed latest ${FILE_NAME}"
fi

backup_name=$(date +%F_%H-%M)
for php_ini_file in $(find /etc -type f -name "php.ini")
do
  if [ -f "${php_ini_file}" ]; then
    upd_log "disable functions in ${php_ini_file}"
    chattr -i "${php_ini_file}"
    sed -e '/^[^;]*disable_functions/s/=.*$/= shell_exec,proc_terminate,pcntl_exec,passthru,proc_nice,dl,show_source,apache_note,apache_setenv,debugger_off,debugger_on,closelog,define_syslog_variables,ini_restore,openlog,syslog,posix_mkfifo,posix_getpwuid,posix_setpgid,posix_uname,posix_setgid,posix_setuid,posix_setegid,posix_getpwuid,posix_setpgid,mail,apache_child_terminate,chgrp,chown,curl_multi_exec,define_syslog_variables,highlight_file,parse_ini_string,stream_select,imap_open,xmlrpc_entity_decode,inject_code,ini_alter,xmlrpc_encode,xmlrpc_decode,xmlrpc_get_type,xmlrpc_is_fault,xmlrpc_set_type,phpAds_remoteInfo,phpAds_XmlRpc,phpAds_xmlrpcDecode,phpAds_xmlrpcEncode,ftp_exec,ftp_get,ftp_nb_fput,ftp_raw,ftp_rawlist,symlink,assert,create_function,phpinfo,posix_ttyname,getenv,posix_getlogin,get_current_user,get_cfg_var,diskfreespace,getlastmo,getmygid,getmyinode,getmyuid,extract,putenv,pfsockopen,base64_decode,parse_ini_file,call_user_func_array,call_user_func/' -i.${backup_name} ${php_ini_file}
    service php-fpm restart > /dev/null 2>&1
  fi
done

exit 0
